<?php
class WC_Gateway_Bank extends WC_Payment_Gateway {

    /**
     * Constructor for the gateway.
     */
    public function __construct() {
		$this->id                 = 'bank';
		$this->name 			  = 'Bank';   	
		
		$this->init();
    }	

    /**
     * Init the class
     */
    function init(){
		$this->icon               = apply_filters( "woocommerce_{$this->id}_icon", '' );
		$this->has_fields         = false;
		$this->method_title       = __( $this->name, 'plugins-wpkonten' );
		$this->method_description = __( "Allows payments using direct bank/wire transfer by {$this->name}.", 'plugins-wpkonten' );

        // Define user set variables
		$this->title        = $this->get_option( 'the_title', sprintf( __( 'Transfer %s', 'plugins-wpkonten' ), $this->name ) );
		$this->description  = $this->get_option( 'the_description', $this->method_description );
		$this->instructions = $this->get_option( 'the_instructions', $this->method_description );

		// Bank BCA account fields shown on the thanks page and in emails
		$this->account_details = get_option( "woocommerce_{$this->id}_accounts",
			array(
				'the_account_name'   => array( 
					'label' => __( 'Account Name', 'plugins-wpkonten' ), 
					'value' => $this->get_option( 'the_account_name', '-' ) 
				),
				'the_account_number' => array(
					'label' => __( 'Number Account', 'plugins-wpkonten' ),
					'value' => $this->get_option( 'the_account_number', '-' )
				)
			)
		);

		// Actions
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
    	add_action( "woocommerce_thankyou_{$this->id}", array( $this, 'thankyou_page' ) );

    	// Customer Emails
    	add_action( 'woocommerce_email_after_order_table', array( $this, 'email_instructions' ), 10, 3 );


		// Load the settings.
		$this->init_form_fields();
		$this->init_settings(); 
    }

    /**
     * Initialise Gateway Settings Form Fields
     */
    public function init_form_fields() {
    	$this->form_fields = array(
			'enabled' => array(
				'title'   => __( 'Enable/Disable', 'plugins-wpkonten' ),
				'type'    => 'checkbox',
				'label'   => __( "Enable {$this->name} Transfer", 'plugins-wpkonten' ),
				'default' => 'yes'
			),
			'the_title' => array(
				'title'       => __( 'Title', 'plugins-wpkonten' ),
				'type'        => 'text',
				'description' => __( 'This title will be seen by the customer upon checkout process', 'plugins-wpkonten' ),
				'default'     => __( "Direct {$this->name} Transfer", 'plugins-wpkonten' ),
				'desc_tip'    => true,
			),
			'the_description' => array(
				'title'       => __( 'Description', 'plugins-wpkonten' ),
				'type'        => 'textarea',
				'description' => __( 'Payment method description that the customer will see on your checkout.', 'plugins-wpkonten' ),
				'default'     => __( 'Make your payment directly into our bank account. Please use your Order ID as the payment reference. Your order won\'t be shipped until the funds have cleared in our account.', 'plugins-wpkonten' ),
				'desc_tip'    => true,
			),
			'the_instructions' => array(
				'title'       => __( 'Instructions', 'plugins-wpkonten' ),
				'type'        => 'textarea',
				'description' => __( 'Instructions that will be added to the thank you page and emails.', 'plugins-wpkonten' ),
				'default'     => '',
				'desc_tip'    => true,
			),
			'the_account_name' => array(
				'title'       => __( 'Account Name', 'plugins-wpkonten' ),
				'type'        => 'text',
				'description' => __( 'This account name is displayed during checkout process and related emails to the customer', 'plugins-wpkonten' ),
				'default'     => __( 'Jonno', 'plugins-wpkonten' ),
				'desc_tip'    => true,
			),
			'the_account_number' => array(
				'title'       => __( 'Account Number', 'plugins-wpkonten' ),
				'type'        => 'text',
				'description' => __( 'This account number is displayed during checkout process and related emails to the customer', 'plugins-wpkonten' ),
				'default'     => __( '0999777444555', 'plugins-wpkonten' ),
				'desc_tip'    => true,
			),
		);
    }

    /**
     * Output for the order received page.
     */
    public function thankyou_page( $order_id ) {
		if ( $this->instructions ) {
        	echo wpautop( wptexturize( wp_kses_post( $this->instructions ) ) );
        }
        $this->bank_details( $order_id );
    }    

    /**
     * Add content to the WC emails.
     *
     * @access public
     * @param WC_Order $order
     * @param bool $sent_to_admin
     * @param bool $plain_text
     * @return void
     */
    public function email_instructions( $order, $sent_to_admin, $plain_text = false ) {
    	if ( ! $sent_to_admin && $this->id === $order->payment_method && 'on-hold' === $order->status ) {
			if ( $this->instructions ) {
				echo wpautop( wptexturize( $this->instructions ) ) . PHP_EOL;
			}
			$this->bank_details( $order->id );
		}
    }

    /**
     * Get bank details and place into a list format
     */
    private function bank_details( $order_id = '' ) {
    	if ( empty( $this->account_details ) ) {
    		return;
    	}

    	$this->account_information();
    }

    /**
     * Process the payment and return the result
     *
     * @param int $order_id
     * @return array
     */
    public function process_payment( $order_id ) {

		$order = new WC_Order( $order_id );

		// Mark as on-hold (we're awaiting the payment)
		$order->update_status( 'on-hold', __( 'Awaiting payment through', 'plugins-wpkonten' ) );

		// Reduce stock levels
		$order->reduce_order_stock();

		// Remove cart
		WC()->cart->empty_cart();

		// Return thankyou redirect
		return array(
			'result' 	=> 'success',
			'redirect'	=> $this->get_return_url( $order )
		);
    }

	/**
	 * If There are no payment fields show the description if set.
	 * Override this in your gateway if you have some.
	 *
	 * @access public
	 * @return void
	 */
	public function payment_fields() {
		if ( $description = $this->get_description() ) {
			echo wpautop( wptexturize( $description ) );
		}

		$this->account_information();
	}

	/**
	 * Display account information
	 * 
	 * @return void
	 */
	function account_information(){
		$icon_plugin_dir = plugins_url( '../images/'.$this->id.'.png', __FILE__ );
    	echo "<h2 class='order_details_title {$this->id}'> <img src='".$icon_plugin_dir."' style='height:25px;margin-right: 10px;display: inline-table;'/>" . sprintf( apply_filters( 'wc_gateway_bank_order_details_title', __( "%s Account", 'plugins-wpkonten' ) ), $this->name ) . "</h2>" . PHP_EOL;

    	$bank_account = apply_filters( "woocommerce_{$this->id}_accounts", $this->account_details );

		$bank_account = (object) $bank_account;

		echo '<ul class="order_details bank_details">' . PHP_EOL;

		foreach ( $bank_account as $field_key => $field ) {
		    if ( ! empty( $field['value'] ) ) {
		    	echo '<li class="' . esc_attr( $field_key ) . '"><span class="label">' . esc_attr( $field['label'] ) . '</span>: <strong>' . wptexturize( $field['value'] ) . '</strong></li>' . PHP_EOL;
		    }
		}

		echo '</ul>';
	}
}